@@ -21,45 +21,37 @@
 //! A [`ProtocolsHandler`] implementation that combines multiple other `ProtocolsHandler`s
 //! indexed by some key.
 
-use crate::NegotiatedSubstream;
 use crate::protocols_handler::{
-    KeepAlive,
-    IntoProtocolsHandler,
-    ProtocolsHandler,
-    ProtocolsHandlerEvent,
-    ProtocolsHandlerUpgrErr,
-    SubstreamProtocol
-};
-use crate::upgrade::{
-    InboundUpgradeSend,
-    OutboundUpgradeSend,
-    UpgradeInfoSend
+    IntoProtocolsHandler, KeepAlive, ProtocolsHandler, ProtocolsHandlerEvent,
+    ProtocolsHandlerUpgrErr, SubstreamProtocol,
 };
+use crate::upgrade::{InboundUpgradeSend, OutboundUpgradeSend, UpgradeInfoSend};
+use crate::NegotiatedSubstream;
 use futures::{future::BoxFuture, prelude::*};
+use libp2p_core::upgrade::{NegotiationError, ProtocolError, ProtocolName, UpgradeError};
 use libp2p_core::{ConnectedPoint, Multiaddr, PeerId};
-use libp2p_core::upgrade::{ProtocolName, UpgradeError, NegotiationError, ProtocolError};
 use rand::Rng;
 use std::{
     cmp,
     collections::{HashMap, HashSet},
     error,
-    fmt,
+    fmt::{self, Debug},
     hash::Hash,
     iter::{self, FromIterator},
     task::{Context, Poll},
-    time::Duration
+    time::Duration,
 };
 
 /// A [`ProtocolsHandler`] for multiple `ProtocolsHandler`s of the same type.
 #[derive(Clone)]
 pub struct MultiHandler<K, H> {
-    handlers: HashMap<K, H>
+    handlers: HashMap<K, H>,
 }
 
 impl<K, H> fmt::Debug for MultiHandler<K, H>
 where
     K: fmt::Debug + Eq + Hash,
-    H: fmt::Debug
+    H: fmt::Debug,
 {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("MultiHandler")
@@ -71,27 +63,33 @@ where
 impl<K, H> MultiHandler<K, H>
 where
     K: Hash + Eq,
-    H: ProtocolsHandler
+    H: ProtocolsHandler,
 {
     /// Create and populate a `MultiHandler` from the given handler iterator.
     ///
     /// It is an error for any two protocols handlers to share the same protocol name.
     pub fn try_from_iter<I>(iter: I) -> Result<Self, DuplicateProtonameError>
     where
-        I: IntoIterator<Item = (K, H)>
+        I: IntoIterator<Item = (K, H)>,
     {
-        let m = MultiHandler { handlers: HashMap::from_iter(iter) };
-        uniq_proto_names(m.handlers.values().map(|h| h.listen_protocol().into_upgrade().0))?;
+        let m = MultiHandler {
+            handlers: HashMap::from_iter(iter),
+        };
+        uniq_proto_names(
+            m.handlers
+                .values()
+                .map(|h| h.listen_protocol().into_upgrade().0),
+        )?;
         Ok(m)
     }
 }
 
 impl<K, H> ProtocolsHandler for MultiHandler<K, H>
 where
-    K: Clone + Hash + Eq + Send + 'static,
+    K: Clone + Debug + Hash + Eq + Send + 'static,
     H: ProtocolsHandler,
     H::InboundProtocol: InboundUpgradeSend,
-    H::OutboundProtocol: OutboundUpgradeSend
+    H::OutboundProtocol: OutboundUpgradeSend,
 {
     type InEvent = (K, <H as ProtocolsHandler>::InEvent);
     type OutEvent = (K, <H as ProtocolsHandler>::OutEvent);
@@ -102,28 +100,31 @@ where
     type OutboundOpenInfo = (K, <H as ProtocolsHandler>::OutboundOpenInfo);
 
     fn listen_protocol(&self) -> SubstreamProtocol<Self::InboundProtocol, Self::InboundOpenInfo> {
-        let (upgrade, info, timeout) = self.handlers.iter()
+        let (upgrade, info, timeout) = self
+            .handlers
+            .iter()
             .map(|(key, handler)| {
                 let proto = handler.listen_protocol();
                 let timeout = *proto.timeout();
                 let (upgrade, info) = proto.into_upgrade();
                 (key.clone(), (upgrade, info, timeout))
             })
-            .fold((Upgrade::new(), Info::new(), Duration::from_secs(0)),
+            .fold(
+                (Upgrade::new(), Info::new(), Duration::from_secs(0)),
                 |(mut upg, mut inf, mut timeout), (k, (u, i, t))| {
                     upg.upgrades.push((k.clone(), u));
                     inf.infos.push((k, i));
                     timeout = cmp::max(timeout, t);
                     (upg, inf, timeout)
-                }
+                },
             );
         SubstreamProtocol::new(upgrade, info).with_timeout(timeout)
     }
 
-    fn inject_fully_negotiated_outbound (
+    fn inject_fully_negotiated_outbound(
         &mut self,
         protocol: <Self::OutboundProtocol as OutboundUpgradeSend>::Output,
-        (key, arg): Self::OutboundOpenInfo
+        (key, arg): Self::OutboundOpenInfo,
     ) {
         if let Some(h) = self.handlers.get_mut(&key) {
             h.inject_fully_negotiated_outbound(protocol, arg)
@@ -132,10 +133,10 @@ where
         }
     }
 
-    fn inject_fully_negotiated_inbound (
+    fn inject_fully_negotiated_inbound(
         &mut self,
         (key, arg): <Self::InboundProtocol as InboundUpgradeSend>::Output,
-        mut info: Self::InboundOpenInfo
+        mut info: Self::InboundOpenInfo,
     ) {
         if let Some(h) = self.handlers.get_mut(&key) {
             if let Some(i) = info.take(&key) {
@@ -160,10 +161,10 @@ where
         }
     }
 
-    fn inject_dial_upgrade_error (
+    fn inject_dial_upgrade_error(
         &mut self,
         (key, arg): Self::OutboundOpenInfo,
-        error: ProtocolsHandlerUpgrErr<<Self::OutboundProtocol as OutboundUpgradeSend>::Error>
+        error: ProtocolsHandlerUpgrErr<<Self::OutboundProtocol as OutboundUpgradeSend>::Error>,
     ) {
         if let Some(h) = self.handlers.get_mut(&key) {
             h.inject_dial_upgrade_error(arg, error)
@@ -175,77 +176,118 @@ where
     fn inject_listen_upgrade_error(
         &mut self,
         mut info: Self::InboundOpenInfo,
-        error: ProtocolsHandlerUpgrErr<<Self::InboundProtocol as InboundUpgradeSend>::Error>
+        error: ProtocolsHandlerUpgrErr<<Self::InboundProtocol as InboundUpgradeSend>::Error>,
     ) {
         match error {
-            ProtocolsHandlerUpgrErr::Timer =>
+            ProtocolsHandlerUpgrErr::Timer => {
                 for (k, h) in &mut self.handlers {
                     if let Some(i) = info.take(k) {
                         h.inject_listen_upgrade_error(i, ProtocolsHandlerUpgrErr::Timer)
                     }
                 }
-            ProtocolsHandlerUpgrErr::Timeout =>
+            }
+            ProtocolsHandlerUpgrErr::Timeout => {
                 for (k, h) in &mut self.handlers {
                     if let Some(i) = info.take(k) {
                         h.inject_listen_upgrade_error(i, ProtocolsHandlerUpgrErr::Timeout)
                     }
                 }
-            ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Select(NegotiationError::Failed)) =>
+            }
+            ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Select(NegotiationError::Failed)) => {
                 for (k, h) in &mut self.handlers {
                     if let Some(i) = info.take(k) {
-                        h.inject_listen_upgrade_error(i, ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Select(NegotiationError::Failed)))
+                        h.inject_listen_upgrade_error(
+                            i,
+                            ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Select(
+                                NegotiationError::Failed,
+                            )),
+                        )
                     }
                 }
-            ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Select(NegotiationError::ProtocolError(e))) =>
-                match e {
-                    ProtocolError::IoError(e) =>
-                        for (k, h) in &mut self.handlers {
-                            if let Some(i) = info.take(k) {
-                                let e = NegotiationError::ProtocolError(ProtocolError::IoError(e.kind().into()));
-                                h.inject_listen_upgrade_error(i, ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Select(e)))
-                            }
-                        }
-                    ProtocolError::InvalidMessage =>
-                        for (k, h) in &mut self.handlers {
-                            if let Some(i) = info.take(k) {
-                                let e = NegotiationError::ProtocolError(ProtocolError::InvalidMessage);
-                                h.inject_listen_upgrade_error(i, ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Select(e)))
-                            }
-                        }
-                    ProtocolError::InvalidProtocol =>
-                        for (k, h) in &mut self.handlers {
-                            if let Some(i) = info.take(k) {
-                                let e = NegotiationError::ProtocolError(ProtocolError::InvalidProtocol);
-                                h.inject_listen_upgrade_error(i, ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Select(e)))
-                            }
-                        }
-                    ProtocolError::TooManyProtocols =>
-                        for (k, h) in &mut self.handlers {
-                            if let Some(i) = info.take(k) {
-                                let e = NegotiationError::ProtocolError(ProtocolError::TooManyProtocols);
-                                h.inject_listen_upgrade_error(i, ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Select(e)))
-                            }
+            }
+            ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Select(
+                NegotiationError::ProtocolError(e),
+            )) => match e {
+                ProtocolError::IoError(e) => {
+                    for (k, h) in &mut self.handlers {
+                        if let Some(i) = info.take(k) {
+                            let e = NegotiationError::ProtocolError(ProtocolError::IoError(
+                                e.kind().into(),
+                            ));
+                            h.inject_listen_upgrade_error(
+                                i,
+                                ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Select(e)),
+                            )
                         }
+                    }
                 }
-            ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Apply((k, e))) =>
+                ProtocolError::InvalidMessage => {
+                    for (k, h) in &mut self.handlers {
+                        if let Some(i) = info.take(k) {
+                            let e = NegotiationError::ProtocolError(ProtocolError::InvalidMessage);
+                            h.inject_listen_upgrade_error(
+                                i,
+                                ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Select(e)),
+                            )
+                        }
+                    }
+                }
+                ProtocolError::InvalidProtocol => {
+                    for (k, h) in &mut self.handlers {
+                        if let Some(i) = info.take(k) {
+                            let e = NegotiationError::ProtocolError(ProtocolError::InvalidProtocol);
+                            h.inject_listen_upgrade_error(
+                                i,
+                                ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Select(e)),
+                            )
+                        }
+                    }
+                }
+                ProtocolError::TooManyProtocols => {
+                    for (k, h) in &mut self.handlers {
+                        if let Some(i) = info.take(k) {
+                            let e =
+                                NegotiationError::ProtocolError(ProtocolError::TooManyProtocols);
+                            h.inject_listen_upgrade_error(
+                                i,
+                                ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Select(e)),
+                            )
+                        }
+                    }
+                }
+            },
+            ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Apply((k, e))) => {
                 if let Some(h) = self.handlers.get_mut(&k) {
                     if let Some(i) = info.take(&k) {
-                        h.inject_listen_upgrade_error(i, ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Apply(e)))
+                        h.inject_listen_upgrade_error(
+                            i,
+                            ProtocolsHandlerUpgrErr::Upgrade(UpgradeError::Apply(e)),
+                        )
                     }
                 }
+            }
         }
     }
 
     fn connection_keep_alive(&self) -> KeepAlive {
-        self.handlers.values()
+        self.handlers
+            .values()
             .map(|h| h.connection_keep_alive())
             .max()
             .unwrap_or(KeepAlive::No)
     }
 
-    fn poll(&mut self, cx: &mut Context<'_>)
-        -> Poll<ProtocolsHandlerEvent<Self::OutboundProtocol, Self::OutboundOpenInfo, Self::OutEvent, Self::Error>>
-    {
+    fn poll(
+        &mut self,
+        cx: &mut Context<'_>,
+    ) -> Poll<
+        ProtocolsHandlerEvent<
+            Self::OutboundProtocol,
+            Self::OutboundOpenInfo,
+            Self::OutEvent,
+            Self::Error,
+        >,
+    > {
         // Calling `gen_range(0, 0)` (see below) would panic, so we have return early to avoid
         // that situation.
         if self.handlers.is_empty() {
@@ -257,15 +299,19 @@ where
 
         for (k, h) in self.handlers.iter_mut().skip(pos) {
             if let Poll::Ready(e) = h.poll(cx) {
-                let e = e.map_outbound_open_info(|i| (k.clone(), i)).map_custom(|p| (k.clone(), p));
-                return Poll::Ready(e)
+                let e = e
+                    .map_outbound_open_info(|i| (k.clone(), i))
+                    .map_custom(|p| (k.clone(), p));
+                return Poll::Ready(e);
             }
         }
 
         for (k, h) in self.handlers.iter_mut().take(pos) {
             if let Poll::Ready(e) = h.poll(cx) {
-                let e = e.map_outbound_open_info(|i| (k.clone(), i)).map_custom(|p| (k.clone(), p));
-                return Poll::Ready(e)
+                let e = e
+                    .map_outbound_open_info(|i| (k.clone(), i))
+                    .map_custom(|p| (k.clone(), p));
+                return Poll::Ready(e);
             }
         }
 
@@ -276,13 +322,13 @@ where
 /// A [`IntoProtocolsHandler`] for multiple other `IntoProtocolsHandler`s.
 #[derive(Clone)]
 pub struct IntoMultiHandler<K, H> {
-    handlers: HashMap<K, H>
+    handlers: HashMap<K, H>,
 }
 
 impl<K, H> fmt::Debug for IntoMultiHandler<K, H>
 where
     K: fmt::Debug + Eq + Hash,
-    H: fmt::Debug
+    H: fmt::Debug,
 {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("IntoMultiHandler")
@@ -291,20 +337,21 @@ where
     }
 }
 
-
 impl<K, H> IntoMultiHandler<K, H>
 where
     K: Hash + Eq,
-    H: IntoProtocolsHandler
+    H: IntoProtocolsHandler,
 {
     /// Create and populate an `IntoMultiHandler` from the given iterator.
     ///
     /// It is an error for any two protocols handlers to share the same protocol name.
     pub fn try_from_iter<I>(iter: I) -> Result<Self, DuplicateProtonameError>
     where
-        I: IntoIterator<Item = (K, H)>
+        I: IntoIterator<Item = (K, H)>,
     {
-        let m = IntoMultiHandler { handlers: HashMap::from_iter(iter) };
+        let m = IntoMultiHandler {
+            handlers: HashMap::from_iter(iter),
+        };
         uniq_proto_names(m.handlers.values().map(|h| h.inbound_protocol()))?;
         Ok(m)
     }
@@ -312,24 +359,28 @@ where
 
 impl<K, H> IntoProtocolsHandler for IntoMultiHandler<K, H>
 where
-    K: Clone + Eq + Hash + Send + 'static,
-    H: IntoProtocolsHandler
+    K: Debug + Clone + Eq + Hash + Send + 'static,
+    H: IntoProtocolsHandler,
 {
     type Handler = MultiHandler<K, H::Handler>;
 
     fn into_handler(self, p: &PeerId, c: &ConnectedPoint) -> Self::Handler {
         MultiHandler {
-            handlers: self.handlers.into_iter()
+            handlers: self
+                .handlers
+                .into_iter()
                 .map(|(k, h)| (k, h.into_handler(p, c)))
-                .collect()
+                .collect(),
         }
     }
 
     fn inbound_protocol(&self) -> <Self::Handler as ProtocolsHandler>::InboundProtocol {
         Upgrade {
-            upgrades: self.handlers.iter()
+            upgrades: self
+                .handlers
+                .iter()
                 .map(|(k, h)| (k.clone(), h.inbound_protocol()))
-                .collect()
+                .collect(),
         }
     }
 }
@@ -347,7 +398,7 @@ impl<H: ProtocolName> ProtocolName for IndexedProtoName<H> {
 /// The aggregated `InboundOpenInfo`s of supported inbound substream protocols.
 #[derive(Clone)]
 pub struct Info<K, I> {
-    infos: Vec<(K, I)>
+    infos: Vec<(K, I)>,
 }
 
 impl<K: Eq, I> Info<K, I> {
@@ -357,7 +408,7 @@ impl<K: Eq, I> Info<K, I> {
 
     pub fn take(&mut self, k: &K) -> Option<I> {
         if let Some(p) = self.infos.iter().position(|(key, _)| key == k) {
-            return Some(self.infos.remove(p).1)
+            return Some(self.infos.remove(p).1);
         }
         None
     }
@@ -366,19 +417,21 @@ impl<K: Eq, I> Info<K, I> {
 /// Inbound and outbound upgrade for all `ProtocolsHandler`s.
 #[derive(Clone)]
 pub struct Upgrade<K, H> {
-    upgrades: Vec<(K, H)>
+    upgrades: Vec<(K, H)>,
 }
 
 impl<K, H> Upgrade<K, H> {
     fn new() -> Self {
-        Upgrade { upgrades: Vec::new() }
+        Upgrade {
+            upgrades: Vec::new(),
+        }
     }
 }
 
 impl<K, H> fmt::Debug for Upgrade<K, H>
 where
     K: fmt::Debug + Eq + Hash,
-    H: fmt::Debug
+    H: fmt::Debug,
 {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("Upgrade")
@@ -390,13 +443,15 @@ where
 impl<K, H> UpgradeInfoSend for Upgrade<K, H>
 where
     H: UpgradeInfoSend,
-    K: Send + 'static
+    K: Send + 'static,
 {
     type Info = IndexedProtoName<H::Info>;
     type InfoIter = std::vec::IntoIter<Self::Info>;
 
     fn protocol_info(&self) -> Self::InfoIter {
-        self.upgrades.iter().enumerate()
+        self.upgrades
+            .iter()
+            .enumerate()
             .map(|(i, (_, h))| iter::repeat(i).zip(h.protocol_info()))
             .flatten()
             .map(|(i, h)| IndexedProtoName(i, h))
@@ -408,21 +463,20 @@ where
 impl<K, H> InboundUpgradeSend for Upgrade<K, H>
 where
     H: InboundUpgradeSend,
-    K: Send + 'static
+    K: Send + 'static,
 {
     type Output = (K, <H as InboundUpgradeSend>::Output);
-    type Error  = (K, <H as InboundUpgradeSend>::Error);
+    type Error = (K, <H as InboundUpgradeSend>::Error);
     type Future = BoxFuture<'static, Result<Self::Output, Self::Error>>;
 
     fn upgrade_inbound(mut self, resource: NegotiatedSubstream, info: Self::Info) -> Self::Future {
         let IndexedProtoName(index, info) = info;
         let (key, upgrade) = self.upgrades.remove(index);
-        upgrade.upgrade_inbound(resource, info)
-            .map(move |out| {
-                match out {
-                    Ok(o) => Ok((key, o)),
-                    Err(e) => Err((key, e))
-                }
+        upgrade
+            .upgrade_inbound(resource, info)
+            .map(move |out| match out {
+                Ok(o) => Ok((key, o)),
+                Err(e) => Err((key, e)),
             })
             .boxed()
     }
@@ -431,21 +485,20 @@ where
 impl<K, H> OutboundUpgradeSend for Upgrade<K, H>
 where
     H: OutboundUpgradeSend,
-    K: Send + 'static
+    K: Send + 'static,
 {
     type Output = (K, <H as OutboundUpgradeSend>::Output);
-    type Error  = (K, <H as OutboundUpgradeSend>::Error);
+    type Error = (K, <H as OutboundUpgradeSend>::Error);
     type Future = BoxFuture<'static, Result<Self::Output, Self::Error>>;
 
     fn upgrade_outbound(mut self, resource: NegotiatedSubstream, info: Self::Info) -> Self::Future {
         let IndexedProtoName(index, info) = info;
         let (key, upgrade) = self.upgrades.remove(index);
-        upgrade.upgrade_outbound(resource, info)
-            .map(move |out| {
-                match out {
-                    Ok(o) => Ok((key, o)),
-                    Err(e) => Err((key, e))
-                }
+        upgrade
+            .upgrade_outbound(resource, info)
+            .map(move |out| match out {
+                Ok(o) => Ok((key, o)),
+                Err(e) => Err((key, e)),
             })
             .boxed()
     }
@@ -455,14 +508,14 @@ where
 fn uniq_proto_names<I, T>(iter: I) -> Result<(), DuplicateProtonameError>
 where
     I: Iterator<Item = T>,
-    T: UpgradeInfoSend
+    T: UpgradeInfoSend,
 {
     let mut set = HashSet::new();
     for infos in iter {
         for i in infos.protocol_info() {
             let v = Vec::from(i.protocol_name());
             if set.contains(&v) {
-                return Err(DuplicateProtonameError(v))
+                return Err(DuplicateProtonameError(v));
             } else {
                 set.insert(v);
             }
